package cds.map.jel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


import nom.tam.fits.Fits;
import nom.tam.fits.FitsException;
import nom.tam.fits.Header;
import nom.tam.fits.HeaderCard;
import nom.tam.fits.ImageHDU;
import nom.tam.util.Cursor;

public class FitsMap {
	private File fitsF;
	private File mapF;
	public FitsMap(File fitsF, File mapF) {
		super();
		this.fitsF = fitsF;
		this.mapF = mapF;
	}
		
	/**
	 * Cette fonction permet de retourner la valeur d'un keyword donn en parametre
	 * 
	 * @param keywd le keyword
	 * @return la valeur du keyword s'il existe, null sinon
	 */
	
	private String valueOfKeyWord(String keywd){
		String retour="null";
		//if(this.objectValide)
		{
			try{
				Fits fi=new Fits(this.fitsF);
				ImageHDU ih=(ImageHDU)fi.readHDU();
				if(fi.getNumberOfHDUs()!=0){				
					Header hdr=ih.getHeader();
					Cursor iter =hdr.iterator();
					while (iter.hasNext()) {
						HeaderCard hc = (HeaderCard) iter.next();
						if(keywd.trim().compareTo(hc.getKey())==0)
						{
							if(hc.getValue()!=null)
							{
								//retour=interpretFitsValues(hc.getValue().trim());
								//System.out.println("1\n"+hc.toString());
								retour=hc.getValue().trim();

							}
							else{//probleme au niveau de la librairie fits qui n'arrive pas
								// a extraire la valeur du keyword
								//Solution
								//on extrait le value du keyword,  l'aide des fonctions
								//toString() et split()
								String[] val=hc.toString().split("'");
								
								retour=val[1].trim();

							}
							break;
						}
//						else
//						return null;
					}	
				}
			}catch(FitsException e){//seule cette exception qui doit etre traite soignesement
				//les autres sont traites au niveau de la creation de l'objet et ne peuvent
				//jamais survenir
				//this.objectValide=false;
				System.out.print("Fits Error: "+e.getMessage());
				//WinGenFmFits.erreur("Fits Error: "+e.getMessage());
				//ecriture du log a prevoir			
			}
			catch(IOException e){
				System.out.println("An I/O error occured: " + e.getMessage());
				//this.objectValide=false;

			}catch (NullPointerException e){
				//this.objectValide=false;
				System.out.println(e.getMessage());			
			}
		}
		return retour;
	}
	/**
	 *Cette fonction permet de gnrer le fichier UTY
	 * @param utypeF: le fichier UTY de sortie
	 */
	 void map(File utypeF){
		try{
			//Lecture du fichier de mappig
			FileReader fr= new FileReader(this.mapF);
			BufferedReader br =new BufferedReader(fr);

			//Ecriture du fichier UTY
			PrintWriter pw = new PrintWriter( new FileWriter(utypeF));
			String line;
			while((line=br.readLine())!=null){
				if(!line.trim().equals("")){					
					//chaque ligne du fichier sera decompose selon le token #
					StringTokenizer st=new StringTokenizer(line,"#");	
				
					while(st.hasMoreTokens()){	
						String key=st.nextToken().trim();
						key=IterpretValue(key);
						String utype=st.nextToken();
						//if(key)
						pw.println("%"+utype.trim()+" "+key.trim());
						System.out.println("%"+utype.trim()+" "+key.trim());
					}
				}
				else{
					System.out.println();
					pw.println();
					}
			}
			fr.close();
			pw.close();
		}catch(IOException ioe){
			System.out.println("An I/O error occured: " + ioe.getMessage());
		}
	}

	 private String IterpretValue(String s){
		 //Elimination des espaces de l'espression
		 s=s.replaceAll("\\s","");
		 //partie statique ~machin
		 if(s.startsWith("~"))
			 return s.substring(1);
		 else
			 if(s.matches("(\\$\\w{1,8})")){// $key
				 return this.onlyKey(s);
			 }
			 else
				 //valeur de la forme ($key1,$key2,.......,$keyN)
				 if(s.matches("(^[(](\\$\\w{1,8},)+(\\$\\w{1,8})[)]$)"))
					 //if(s.matches("(\\$\\w{1,8})"))
					 return replaceKeyByValueInExpression(s);
				 else
					 //valeur de la forme 
					 //((min(Va1,Va2,...,Van);min(Vb1,Vb2,...,Vbn))..((max(Vc1,Vc2,...,Vcn);max(Vc1,Vc2,...,Vcn))
					 if(s.matches("^(\\(min\\()(.)+\\);min\\((.)+\\)\\.\\.(\\(max\\()(.)+\\);max\\((.)+\\)")){
						 return coordInterval(s);
					 }
					 else
						 //$key1..$key2//je dois rajouter le fait de support des expressions
						 if(s.matches("^(\\$\\w{1,8})\\.\\.(\\$\\w{1,8})"))
						 {
							 return replaceKeyByValueInExpression(s);
						 }
						 else 
							 //structure de choix
							 if(s.matches("^(\\?\\(\\$\\w{1,8})(\\|\\|(\\$\\w{1,8}))*(\\)):\\(@(\\w{1,8}),(.)+\\)(\\|\\(@(\\w{1,8}),(.)+\\))*"))
							 {

								 return choice(s);

							 }
							 else

							 {
								 s=replaceKeyByValueInExpression(s);
								 //System.out.println(s);

								 //s=ExpressionJep.computeExpression(s);
								 //System.out.println(s);
								 s=ExpressionsJel.computeExpression(s);
								 return s;

							 }
		 //return s;
	 }
	 
	 private String onlyKey(String key){
		 
		 String vKey=valueOfKeyWord(key.substring(1));
		 return this.interpretFitsValues(vKey);
	 }
	 
	private String coordInterval(String str){
		
			
		str=replaceKeyByValueInExpression(str);
		//Decomposition selon  ".."
		String t1[]=str.split("\\.\\.");
		//System.out.println("taille t1:"+ t1.length);

		//System.out.println("decomp selon ':'\n"+t1[0] +"\n"+t1[1]);
		//Decomposition de la 1ere partie selon ";"  partie des min
		String t2[]=t1[0].split(";");
		//System.out.println("taille t2 :"+ t2.length);
		//Elimination de la 1ere parenthes
		
		t2[0]=t2[0].replaceFirst("\\(","");
		/////
		 //System.out.println(t2[0]);
		//t2[0]=ExpressionJep.computeExpression(t2[0]);
		t2[0]=ExpressionsJel.computeExpression(t2[0]);
		////
		//System.out.println("Elimination du 1er (\n"+t2[0]);
		int li=t2[1].lastIndexOf(')');
		//System.out.println("Indice du dernier ) :"+li);
		t2[1]=t2[1].substring(0,li);
		/////
		//System.out.println(t2[1]);
		//t2[1]=ExpressionJep.computeExpression(t2[1]);
		t2[1]=ExpressionsJel.computeExpression(t2[1]);
		////
		//System.out.println("2eme min"+t2[1]);
		
		//System.out.println("("+t2[0]+","+t2[1]+")");
		////////////////////////////////////
		//Decomposition de la 1ere partie selon ";"  partie des min
		String t3[]=t1[1].split(";");
//Elimination de la 1ere parenthes
		
		t3[0]=t3[0].replaceFirst("\\(","");
		/////
		//System.out.println(t3[0]);
		//t3[0]=ExpressionJep.computeExpression(t3[0]);
		t3[0]=ExpressionsJel.computeExpression(t3[0]);
		////
		//System.out.println("Elimination du 1er (\n"+t3[0]);
		int limax=t3[1].lastIndexOf(')');
		//System.out.println("Indice du dernier ) :"+limax);
		t3[1]=t3[1].substring(0,limax);
		/////
		//System.out.println(t3[1]);
		//t3[1]=ExpressionJep.computeExpression(t3[1]);
		t3[1]=ExpressionsJel.computeExpression(t3[1]);
		////
		//System.out.println("2eme min"+t3[1]);
		
		//System.out.println("("+t3[0]+","+t3[1]+")");
		
		str="("+t2[0]+","+t2[1]+")"+".."+"("+t3[0]+","+t3[1]+")";
		//System.out.println(str);
		
		return str;
	}
	/**
	 * Cette fonction permet de substituer les Keywords de l'expression passe 
	 * en parametres par leurs valeurs. Les valeurs sont extraites  partie de l'entte du fichier FITS
	 * @param exp
	 * @param fitsFile
	 * @return l'expression avec les valeurs
	 */
	private String replaceKeyByValueInExpression(String exp){

		String out="";
		//on decompose la chaine selon le caractaire $, ce dernier precede chaque keyWord
		String a[]=exp.split("\\$");

		for(int i=0;i<a.length;i++)
		{
			a[i]=a[i].trim();
			//si la sous chaine ne commence pas par un symbole de ponctuation
			if(a[i].matches("^[^\\p{Punct}]+.+"))

			{
				Pattern p=Pattern.compile("\\w{1,8}");
				a[i].trim();
				Matcher m = p.matcher(a[i]);
				m.find();
				int indexEndKey=m.end();
				String key=a[i].substring(0,indexEndKey).trim();
				String vKey=valueOfKeyWord(key);
				if(vKey!=null &&  vKey!="null"){
					//if(vKey.startsWith("-"))
					//	vKey="("+vKey+")";
					key=vKey;
				}

				String sdPart=a[i].substring(indexEndKey);

				out=out+key+sdPart;
			}
			else
				out=out+a[i];
		}
		return out.replaceAll("\\s","");
	}
	/**
	 * Juste pour tester les expressions regulieres
	 */
	
	public void testExpression(){
		/*
		String key = "\\w{1,8}";
		String mot1 = "^[(][\\$\\w{1,8},]*[\\$\\w{1,8})]$";
		String mot = "^[(](\\$\\w{1,8},)+(\\$\\w{1,8})[)]$";
		
		String alter="^[\\?](\\(\\$\\w{1,8})(\\|\\|\\$\\w{1,8})*\\):" +
				"\\(F435W,.+\\)\\|\\(F775W,.+\\)\\|\\(F850LP,.+\\)";
		
		//String motif="\\([Mm]in\\(.+\\)";
		//String mot2="(\\({0,1}(\\p{Digit}+)(\\*|/|\\+\\-){0,1})+";
		//String mot3="^(\\(min\\()(.)+\\);min\\((.)+\\)\\.\\.(\\(max\\()(.)+\\);max\\((.)+\\)";
		//String mot3="^(\\$\\w{1,8})..(\\$\\w{1,8})";
		
		//String mot3="^(\\?\\(\\$\\w{1,8})(\\|\\|(\\$\\w{1,8}))*(\\)):\\(@(\\w{1,8}),(.)+\\)(\\|\\(@(\\w{1,8}),(.)+\\))*";
		
		String chaine = "?($FILTER1||$FILTER2):(F435W,(2.4e06/0.435)*(180/PI))|(F606W,(2.4e06/0.606)*(180/PI))|(F775W,(2.4e06/0.775)*(180/PI))|(F850LP,(2.4e06/0.850)*(180/PI))";
		//String chaine1 = "(min(2+1,A,3),min(2,AZ,EE,Z))..(max(2,A,3),max(2,AZ,EE,Z))";
		//String chaine1 = "(min($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*-($CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ); min($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))..( max($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ); max($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))";
		//String chaine1="($FILTER1||$FILTER2||$FILTER2):(@F435W,(2.4e06/0.435)*(180/PI))";
		//String chaine1="?($FILTER1||$FILTER2):(@F435W,(2.4e06/0.435)*(180/pi))|(@F606W,(2.4e06/0.606)*(180/pi))|(@F775W,(2.4e06/0.775)*(180/pi))|(@F850LP,(2.4e06/0.850)*(180/pi))";
		String chaine2="?($FILTER1||$FILTER2):(@F435W,(2.4e06/0.435)*(180/pi))|(@F606W,(2.4e06/0.606)*(180/pi))|(@F775W,(2.4e06/0.775)*(180/pi))|(@F850LP,(2.4e06/0.850)*(180/pi))";
		//chaine1=replaceKeyByValueInExpression(chaine1,fitsFile);
		
//		chaine1=chaine1.replaceAll("\\s","");
		//chaine1=chaine1.replaceAll("\\s","");
		//System.out.println(chaine1);
		String chaine1="$mot";
		String mot3="(\\$\\w{1,8})";
		
		
		
		if(chaine1.matches(mot3) ){
		
		    System.out.println("chaine correcte");
		
		}else{
		
		    System.out.println("Autre");
		
		}
		
		
		*/
	}
	
	public void testExpr(){
		String expr="atan(($CD1_1*($NAXIS1/2-$CRPIX1)+$CD1_1*($NAXIS2/2-$CRPIX2)/(cos($CRVAL2)-($CD1_1*($NAXIS1/2-$CRPIX1)+$CD1_1*($NAXIS2/2-$CRPIX2)* sin($CRVAL2)))))";
		expr=this.replaceKeyByValueInExpression(expr);
		//expr=ExpressionJep.computeExpression(expr);
		expr=ExpressionsJel.computeExpression(expr);
		//System.out.println(expr);
	}
	
	
	
	/**
	 * Not ready: Permet d'evaluer une stricture de choix
	 * @param s
	 * @return
	 */
	private String choice(String s){
		
		s=s.replaceAll("\\s","");
		s=replaceKeyByValueInExpression(s);
		//on divise la chaine en 2
		String t1[]=s.split(":");
		//decomposition de t1[0] pour recuperer les keyword a tester

		String t2[]=t1[0].trim().split("[(\\?\\()(\\|\\|)(\\))]");
		//System.out.println(t1[0]);
		t2=suppVide(t2);
		//on teste si le fichier FITS contient les KeyWords de l'expression
		
//		for(int i=0;i<t2.length;i++){
//			//if(t2[i].compareTo("null")==0)
//			{
//				System.out.println("HHHHHHHHHHHHHHHHH  "+t2[i]);
//				//return "null";
//			}
//		}
//		
		
		
		
		
		
		String t3[]=t1[1].split("\\|");
		HashMap<String, String> hm=new HashMap<String, String>();
		for(int i=0;i<t3.length;i++){

			t3[i]=t3[i].replaceFirst(",",";");
			String t4[]=t3[i].split(";");
			t4[0]=t4[0].replace("(@","");
			//System.out.println(t4[0]);
			
			int limax=t4[1].lastIndexOf(')');
			//System.out.println("Indice du dernier ) :"+limax);
			t4[1]=t4[1].substring(0,limax);
			if(!t4[1].matches("(.)+\\.\\.(.)+"))
				//System.out.println(t4[1]);
				//t4[1]=ExpressionJep.computeExpression(t4[1]);
				t4[1]=ExpressionsJel.computeExpression(t4[1]);
			hm.put(t4[0],t4[1]);

			//System.out.println(t4[1]);
		}

		for(int i=0;i<t2.length;i++){
			if(hm.containsKey(t2[i]))
			{
				//System.out.println(t2[i]+"\t"+hm.get(t2[i]));
				return hm.get(t2[i]);
				
			}
		}
		
		return "null";
	}
	/**
	 * Cette fonction permet de supprimer les elements vides d'un tableau de String
	 * donn en parametre
	 * @param tab
	 * @return 
	 */
	public static String[] suppVide(String [] tab){
		String [] t= null;//new String(3);
		int taille=0;
		Vector<String> vect=new Vector<String>();
		//on cherche la taille du tableau t
		for(int i=0;i<tab.length;i++)
			if(tab[i].trim().length()!=0)
				taille++;
		t=new String[taille];

		for(int i=0;i<tab.length;i++)
			if(tab[i].trim().length()!=0)
				vect.add(tab[i]);
		for(int i=0;i<vect.size();i++)
			t[i]=vect.elementAt(i);
		return t;
	}
	/**
	 * Cette fonction permet de formater la valeur de la chaine de caractere donne
	 * en parametre representant des valeurs a plusiurs dimensions qu'on extrait du 
	 * headerFits de l'image,selon le format suport par la libraire characterisationLibrary
	 * @param string
	 * @return la chaine formate
	 */
	private String interpretFitsValues(String string){
		string=string.trim();
		String retour=string;
		String[] val=string.split("\\s");
		//System.out.println("s: "+val.length+"   "+val[0]);

		if(val.length>=2 ){
			retour="(";
			for(int i=0;i<val.length;i++ ){
				if(val[i].trim().length()==0)
					continue;
				retour=retour+val[i];
				if(i<val.length-1)
					retour=retour+",";
			}
			retour=retour+")";
		}
		else//string ne contient pas \s
		{
			val =string.split("-");
			//System.out.println("\n-: "+val.length +" "+val[1]);
			//if(val[0].trim()=="")
			if(val.length==2 && (val[0].trim().length()!=0)){
				retour="(";
				for(int i=0;i<val.length;i++ ){
					if(val[i].trim().length()==0)
						continue;
					if(i==0)
						retour=retour+val[i];
					else
						retour=retour+"-"+val[i];
					if(i<val.length-1)
						retour=retour+",";
				}
				retour=retour+")";
			}
			else{
				//pour ne pas spliter des chaines de la forme 3.4534E+3
				if(!string.trim().contains("E+")&&!string.trim().contains("e+"))
					val=string.split("\\+");
			
				if(val.length>=2 && (val[0].trim().length()!=0)){
					retour="(";
					for(int i=0;i<val.length;i++ ){
						if(val[i].trim().length()==0)
							continue;
						retour=retour+val[i];
						if(i<val.length-1)
							retour=retour+",";
					}
					retour=retour+")";
				}

			}

		}

		return retour;
	}
	
	public static void main(String[] args) {
	
//		File f=new File("./HeadFits/3.fits");
//		File map=new File("./HeadFits/map.map");
//		File uty=new File("./HeadFits/3.uty");
//		FitsMap fm=new FitsMap(f,map);
//		//String a[] =fitsKeyWords(f);
//		String chaine = "?($FILTER1||$FILTER2):(F435W,(2.4e06/0.435)*(180/PI))|(F606W,(2.4e06/0.606)*(180/PI))|(F775W,(2.4e06/0.775)*(180/PI))|(F850LP,(2.4e06/0.850)*(180/PI))";
//
//		//System.out.println(valueOfKeyWord("FILTER1",f));
//		//System.out.println(IterpretValue("~key"));
//		String t1="$NAXIS1";
//		String t2="min($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*-($CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1). ";
//		String t3="min($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*-($CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ),Min($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))";
//		String t4="(min($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*-($CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ); Min($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))..( Max($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1-$CRPIX1)+$CD1_2*(-$CRPIX2)+$CRVAL1,$CD1_1*($NAXIS1-$CRPIX1)+$CD1_2*($NAXIS2-$CRPIX2)+$CRVAL1);Max($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2)+$CRVAL2,$CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 )";
//		String t5="max($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1-$CRPIX1)+$CD1_2*(-$CRPIX2)+$CRVAL1,$CD1_1*($NAXIS1-$CRPIX1)+$CD1_2*($NAXIS2-$CRPIX2)+$CRVAL1)";
//		String out=fm.replaceKeyByValueInExpression(t5);
//		String chaine1 = "(min($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*-($CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ); min($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))..( max($CD1_1*(-$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1, $CD1_1*(-$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1, $CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*(-$CRPIX2) +$CRVAL1 ,$CD1_1*($NAXIS1 -$CRPIX1)+$CD1_2*($NAXIS2 -$CRPIX2) +$CRVAL1 ); max($CD2_1*(-$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2, $CD2_1*(-$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2, $CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*(-$CRPIX2) +$CRVAL2 ,$CD2_1*($NAXIS1 -$CRPIX1)+$CD2_2*($NAXIS2 -$CRPIX2) +$CRVAL2 ))";
//		//String ch1="($FILTER1||$FILTER2):(@F435W,(2.4e06/0.435)*(180/pi))|(@F606W,(2.4e06/0.606)*(180/pi))|(@F775W,(2.4e06/0.775)*(180/pi))|(@F850LP,(2.4e06/0.850)*(180/pi))";
//		String ch1="?($FILTER1||$FILTER2):(@F435W,(2.4e06/0.435)*(180/pi))|(@F606W,(2.4e06/0.606)*(180/pi))|(@F775W,(2.4e06/0.775)*(180/pi))|(@F850LP,(2.4e06/0.850)*(180/pi))";
//		//System.out.println(out);
//		//maxN(t5,f);
//		
//		//fm.testExpression();
//		//choice(ch1,f);
//		//minN(t4,f);
//		//maxExpr(t3,f);
//		//coordInterval(chaine1,f);
//		
//		fm.map(uty);
//		//fm.testFile(uty);
//		//fm.testExpr();
//	
	}

}
